@@ -1,7 +1,6 @@
 from typing import Dict, List
 
 from openrtdynamics2 import Signal as sig
-#import .Signal as sig
 
 ORTD_DATATYPE_UNCONFIGURED = 0
 ORTD_DATATYPE_FLOAT = (1 | (8 << 5))
@@ -71,10 +70,10 @@ class DataType(object):
         return True
 
     def show(self):
-        print("Datatype: type=" + self.toStr())
+        print("datatype: type=" + self.toStr())
 
     def toStr(self):
-        return "type=" + str(self.type) + " size=" + str(self.size) + ' (' + self.cppDataType + ')'
+        return self.cppDataType + " [" + str(self.size) + "]"
 
     @property
     def cppDataType(self):
