@@ -4,6 +4,8 @@ from typing import Dict, List
 from . import lang as dy
 import numpy as np
 
+from .BlockPrototypes import *
+
 #
 # constants
 #
@@ -45,6 +47,20 @@ def boolean(value : int):
 
 
 
+#
+#
+#
+
+def initial_event():
+    """
+        Fires an event of the first time instant after the reset of the system
+    """
+
+    # TODO: introduce caching like done for counter()
+
+    return dy.counter() == int32(0)
+
+
 #
 # Delay - the basis for all dynamic elements
 #
@@ -65,14 +81,37 @@ def delay(u , initial_state = None):
 
     else:
 
-        event_on_first_sample = dy.counter() == int32(0)  # TODO: introduce a function dy.init_impulse()
+        event_on_first_sample = initial_event()
 
         delayed_input = dy.delay__( u, None )
         delayed_input = dy.conditional_overwrite( delayed_input, event_on_first_sample, initial_state )
 
         return delayed_input
 
+def sample_and_hold(u, event, initial_state = None):
+    """
+        sample & hold
 
+        Samples the input when event is true and hold this value for the proceeding time instants. 
+
+        u             - the input
+        event         - the trigger signal to perform the sampling
+        initial_state - the initial output
+
+        return values
+
+        the sampled input
+
+    """
+
+    # NOTE: this could be implemented in a more comp. efficient way directly in C in BlockPrototypes.py
+
+    y = dy.signal()
+
+    delayed_y = delay( y, initial_state )
+    y << dy.conditional_overwrite( delayed_y, event, u )
+
+    return y
 
 #
 # static functions
@@ -117,7 +156,7 @@ def unwrap_angle(angle, normalize_around_zero = False):
 
 
 
-def saturate(u, lower_limit, uppper_limit):
+def saturate(u, lower_limit = None, uppper_limit = None):
     """
         saturate the input signal
 
@@ -127,8 +166,13 @@ def saturate(u, lower_limit, uppper_limit):
         uppper_limit  - upper bound for the output
     """
 
-    y = dy.conditional_overwrite( u, u < float64(lower_limit), lower_limit )
-    y = dy.conditional_overwrite( y, y > float64(uppper_limit), uppper_limit )
+    y = u
+
+    if lower_limit is not None:
+        y = dy.conditional_overwrite( y, y < float64(lower_limit), lower_limit )
+    
+    if uppper_limit is not None:
+        y = dy.conditional_overwrite( y, y > float64(uppper_limit), uppper_limit )
 
     return y
 
@@ -147,25 +191,25 @@ def rate_limit( u, Ts, lower_limit, uppper_limit, initial_state = 0 ):
     y = dy.signal()
 
     omega = u - y
-    omega_sat = saturate(omega, lower_limit * Ts_, uppper_limit * Ts_)
+    omega_sat = saturate(omega, float64(lower_limit) * Ts_, float64(uppper_limit) * Ts_)
     y << euler_integrator( omega_sat, 1, initial_state=initial_state)
 
     return y
 
 
-def rate_limit_2nd( u, Ts, lower_limit, uppper_limit, gain, initial_state = 0 ):
+# def rate_limit_2nd( u, Ts, lower_limit, uppper_limit, gain, initial_state = 0 ):
 
-    Ts_ = float64(Ts)
+#     Ts_ = float64(Ts)
 
-    y = dy.signal()
+#     y = dy.signal()
 
-    e = ( u - y )
-    omega = dy.tan( u - y ) * dy.float64(gain)
-    omega_sat = saturate(omega, lower_limit * Ts_, uppper_limit * Ts_)
+#     e = ( u - y )
+#     omega = dy.tan( u - y ) * dy.float64(gain)
+#     omega_sat = saturate(omega, lower_limit * Ts_, uppper_limit * Ts_)
 
-    y << euler_integrator(  omega_sat, 1, initial_state=initial_state)
+#     y << euler_integrator(  omega_sat, 1, initial_state=initial_state)
 
-    return y
+#     return y
 
 
 
@@ -174,7 +218,7 @@ def rate_limit_2nd( u, Ts, lower_limit, uppper_limit, gain, initial_state = 0 ):
 #
 
 # TODO: mark as private
-class Counter():
+class __Counter():
     """
         This class is meant to store the counter output signal as it might be used
         by more than one destination block. The instance of this class is per simulation
@@ -215,7 +259,7 @@ def counter():
         tmp.set_name('shared_couter')
 
         # store the output signal of the counter as it might be used again. 
-        dy.get_simulation_context().components['counter'] = Counter(tmp)
+        dy.get_simulation_context().components['counter'] = __Counter(tmp)
 
     else:
         # use the output of an already created counter
@@ -290,15 +334,15 @@ def counter_triggered( upper_limit, stepwidth=None, initial_state = 0, reset=Non
     return counter
 
 
-def toggle(trigger, initial_state=False):
+# def toggle(trigger, initial_state=False):
 
-    state = dy.signal()
+#     state = dy.signal()
 
-    state << dy.flipflop( dy.logic_and( dy.logic_not( state ), trigger ),  dy.logic_and( dy.logic_not( trigger ), state ), 
-                            initial_state=initial_state, 
-                            nodelay=False)
+#     state << dy.flipflop( dy.logic_and( dy.logic_not( state ), trigger ),  dy.logic_and( dy.logic_not( trigger ), state ), 
+#                             initial_state=initial_state, 
+#                             nodelay=False)
 
-    return state
+#     return state
 
 
 
@@ -656,6 +700,14 @@ def transfer_function_discrete(u : dy.Signal, num_coeff : List[float], den_coeff
 #
 
 def PID_controller(r, y, Ts, kp, ki = None, kd = None):
+    """
+        discrete-time PID-controller
+
+        r           - the reference signal
+        y           - the measured plant output
+        Ts          - the sampling time
+        kp, ki, kd  - the controller parameters (proportional, integral, differential)
+    """
     Ts = dy.float64(Ts)
 
     # control error
